
# TODO: Support additional selinux types
%global selinux_variants targeted
%{!?_selinux_policy_version: %global _selinux_policy_version %(%{__sed} -e 's,.*selinux-policy-\\([^/]*\\)/.*,\\1,' /usr/share/selinux/devel/policyhelp 2>/dev/null)}
%global modulename automation-controller

%global tower_version 4.0.0
%global tower_release 1
%global aap_version 2.0.0
%global sdist_tar_name automation-controller-4.0.0-1
%global sdist_source_name automation-controller-4.0.0

%global distro_python_sitelib %(/usr/libexec/platform-python -c "from distutils.sysconfig import get_python_lib; print(get_python_lib())")

# Remove brp-python-bytecompile from os_install_post
%global __os_install_post /usr/lib/rpm/brp-compress \
  %{!?__debug_package:/usr/lib/rpm/brp-strip %{__strip}} \
  /usr/lib/rpm/brp-strip-static-archive %{__strip} \
  /usr/lib/rpm/brp-strip-comment-note %{__strip} %{__objdump} \
  /usr/lib/rpm/brp-python-hardlink

Name: automation-controller
Summary: REST api, server, plugins, UI, & CLI client for ansible
Version: %{tower_version}
Release: %{tower_release}%{?dist}
Source0: https://github.com/ansible/tower/archive/%{version}.tar.gz?/%{sdist_tar_name}.tar.gz
Source1: %{modulename}.te
Source2: automation-controller.service
Source3: %{modulename}.fc
Source4: ansible-automation-platform-setup-%{aap_version}-1.tar.gz
Source5: %{name}-cli-%{version}-1.tar.gz
Source6: rsyslog.conf
%if "%{packager}" == "Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>"
Source7: automation-controller-4.0.0-1-python-deps.tar.gz
%endif
Provides: automation-controller = %{version}-%{release}
Obsoletes: ansible-tower <= %{version}-%{release}

Group: Development/Libraries
License: Commercial, see https://www.redhat.com/en/about/red-hat-end-user-license-agreements and https://www.redhat.com/en/about/licenses
Url: http://ansible.github.com

# Build Requirements
BuildArch: x86_64

BuildRequires: python38
BuildRequires: python38-devel
BuildRequires: python38-pip

BuildRequires: checkpolicy
BuildRequires: selinux-policy-devel
BuildRequires: hardlink
BuildRequires: sed
BuildRequires: git
BuildRequires: make
BuildRequires: gcc
BuildRequires: bzip2
BuildRequires: postgresql-devel
BuildRequires: libffi-devel
BuildRequires: openssl-devel
BuildRequires: openldap-devel
BuildRequires: libxml2-devel
BuildRequires: libxslt-devel
BuildRequires: xmlsec1-devel, xmlsec1-openssl-devel
BuildRequires: libtool-ltdl-devel
BuildRequires: swig
BuildRequires: libyaml-devel

Requires: automation-controller-server = %{version}-%{release}
Requires: automation-controller-ui = %{version}-%{release}


%description
REST api, server, plugins, Web UI, & CLI client for ansible.

%package server
Summary: Automation Controller REST API
AutoReqProv: no
Obsoletes: ansible-tower-server <= %{version}-%{release}
Requires: automation-controller-venv-tower = %{version}-%{release}
Requires: nginx >= 1:1.18.0
Requires: crontabs
Requires: helm
Requires: sshpass
Requires: supervisor >= 3.0
Requires: git
Requires: git-lfs
Requires: subscription-manager
Requires: subversion
Requires: postgresql
Requires: redis
Requires: sos
Requires: rsync
Requires: rsyslog >= 8.1911
Requires: xmlsec1
Requires: xmlsec1-openssl
Requires: libcgroup-tools
Requires: podman
Requires: crun

# SELinux requirements
%if "%{_selinux_policy_version}" != ""
Requires: python3-libsemanage
%endif

Requires(post): policycoreutils
Requires(post): libselinux-utils
Requires(postun): policycoreutils

%description server
Automation Controller Server and REST API

%package ui
Summary: Automation Controller UI
Requires: automation-controller-server = %{version}-%{release}
Obsoletes: ansible-tower-ui <= %{version}-%{release}

%description ui
UI for Automation Controller

%package setup
Summary: Installer for Automation Controller

%description setup
Installer for Automation Controller

%package venv-tower
Summary: Automation Controller virtual environment
AutoReqProv: no
Obsoletes: ansible-tower-venv-ansible <= %{version}-%{release}
Obsoletes: ansible-tower-venv-tower <= %{version}-%{release}
Requires(pre): python38

%description venv-tower
Automation Controller virtual environment

%package cli
Summary: Package for Automation Controller CLI
AutoReqProv: no
Obsoletes: ansible-tower-cli <= %{version}-%{release}
Requires: python38
Requires: python38-setuptools
Requires: python38-requests
Requires: python38-pyyaml

%description cli
Package for Automation Controller CLI

%prep
%setup -q -n %{sdist_source_name}

tar -xf %{SOURCE4}

mkdir SELinux
cp -p %{SOURCE1} %{SOURCE3} SELinux/

%if "%{packager}" == "Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>"
tar -xf %{SOURCE7}
%endif


%build
# Disable debuginfo packages
%define _enable_debug_package 0
%define debug_package %{nil}


%install

export LANG=en_US.UTF-8 LANGUAGE=en_US.en LC_ALL=en_US.UTF-8

mkdir -p %{buildroot}/var/lib/awx
mv ansible-automation-platform-setup-%{aap_version}-1/ %{buildroot}/var/lib/awx/setup

mkdir -p %{buildroot}/var/lib/awx/rsyslog
mkdir -p %{buildroot}/var/lib/awx/rsyslog/conf.d
cp %{SOURCE6} %{buildroot}/var/lib/awx/rsyslog/rsyslog.conf

export PYTHON=python3.8

# Create folder for the virtualenvs
export VENV_BASE=venv
mkdir -p $VENV_BASE

# Fix m2crypto builds (details at http://pkgs.fedoraproject.org/cgit/m2crypto.git/tree/m2crypto.spec)
export SWIG_FEATURES="-cpperraswarn"

%if "%{packager}" == "Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>"
export PIP_OPTIONS="--find-links=tower/requirements/vendor --no-index"

cat >> setup.cfg <<EOF
[easy_install]
allow_hosts = ''
find_links = file://$PWD/tower/requirements/vendor/
EOF

cp setup.cfg ~/.pydistutils.cfg
%endif

# Install the awxkit package
python3.8 -m pip install %{SOURCE5} --root %{buildroot} --no-deps

export CFLAGS="-DXMLSEC_NO_SIZE_T"
# Install the Tower venv and requirements
%{__make} requirements_awx

python3.8 setup.py build

# Build SELinux policy package (.pp)
pushd SELinux
for selinuxvariant in %{selinux_variants}; do
    make NAME=${selinuxvariant} -f /usr/share/selinux/devel/Makefile
    mv %{modulename}.pp %{modulename}.pp.${selinuxvariant}
    make NAME=${selinuxvariant} -f /usr/share/selinux/devel/Makefile clean
done
popd

# Move site-packages to the buildroot
mkdir -p %{buildroot}/var/lib/awx/venv/awx/lib/python3.8
mv venv/awx/lib/python3.8/site-packages %{buildroot}/var/lib/awx/venv/awx/lib/python3.8/site-packages

# Move bin files
mkdir -p %{buildroot}/var/lib/awx/venv/awx/bin
rm -f %{buildroot}/var/lib/awx/venv/awx/bin/pip*
cp -rp venv/awx/bin/pip* %{buildroot}/var/lib/awx/venv/awx/bin/
mv venv/awx/bin/uwsgi %{buildroot}/var/lib/awx/venv/awx/bin/uwsgi
mv venv/awx/bin/uwsgitop %{buildroot}/var/lib/awx/venv/awx/bin/uwsgitop
mv venv/awx/bin/daphne %{buildroot}/var/lib/awx/venv/awx/bin/daphne
mv venv/awx/bin/ansible-runner %{buildroot}/var/lib/awx/venv/awx/bin/ansible-runner
mv venv/awx/bin/receptorctl %{buildroot}/var/lib/awx/venv/awx/bin/receptorctl

# Replace python interpreter shebang to known destination system paths
PYTHON_INTERPRETER='#!/var/lib/awx/venv/awx/bin/python'
sed -i "1 s|^.*$|$PYTHON_INTERPRETER|" %{buildroot}/var/lib/awx/venv/awx/bin/pip*

PYTHON_INTERPRETER='#!/usr/bin/awx-python'
sed -i "1 s|^.*$|$PYTHON_INTERPRETER|" %{buildroot}/var/lib/awx/venv/awx/bin/daphne

PYTHON_INTERPRETER='#!/usr/bin/awx-python'
sed -i "1 s|^.*$|$PYTHON_INTERPRETER|" %{buildroot}/var/lib/awx/venv/awx/bin/ansible-runner

PYTHON_INTERPRETER='#!/usr/bin/awx-python'
sed -i "1 s|^.*$|$PYTHON_INTERPRETER|" %{buildroot}/var/lib/awx/venv/awx/bin/receptorctl

# Install to the buildroot (non virtualenv)
mkdir -p %{buildroot}/rpm-tmp

python3.8 setup.py install --skip-build --no-compile --root=%{buildroot}/rpm-tmp

# Move files from rpm-tmp folder to the appropriate base folders
mv %{buildroot}/rpm-tmp/usr/lib/python3.8/site-packages/* %{buildroot}/var/lib/awx/venv/awx/lib/python3.8/site-packages

mkdir -p %{buildroot}/usr/bin
mv %{buildroot}/rpm-tmp/usr/bin/* %{buildroot}/usr/bin

# Symlinks to old tower-python and tower-manage executables
ln -s /usr/bin/awx-manage %{buildroot}/%{_bindir}/tower-manage
ln -s /usr/bin/awx-python %{buildroot}/%{_bindir}/tower-python

mkdir -p %{buildroot}/usr/share/sosreport/sos/plugins
mv %{buildroot}/rpm-tmp/usr/share/sosreport/sos/plugins/* %{buildroot}/usr/share/sosreport/sos/plugins

mkdir -p %{buildroot}/usr/share/doc/ansible-tower
mv %{buildroot}/rpm-tmp/usr/share/doc/awx/* %{buildroot}/usr/share/doc/ansible-tower

mkdir -p %{buildroot}/lib/systemd/system
install -m 644 %{SOURCE2} %{buildroot}/lib/systemd/system

mkdir -p %{buildroot}/usr/share/awx
mv %{buildroot}/rpm-tmp/usr/share/awx/* %{buildroot}/usr/share/awx

mkdir -p %{buildroot}/var/lib/awx
mv %{buildroot}/rpm-tmp/var/lib/awx/* %{buildroot}/var/lib/awx

rm -rf %{buildroot}/rpm-tmp

# Create other needed folders
install -d %{buildroot}/%{_sysconfdir}/tower/
install -d -m 0750 %{buildroot}/var/log/tower
install -d -m 0750 %{buildroot}/var/lib/awx/job_status
install -d %{buildroot}/%{_datadir}/awx
install -d -m 2775 %{buildroot}/var/run/tower

# SELinux setup
for selinuxvariant in %{selinux_variants}
do
  install -d %{buildroot}%{_datadir}/selinux/${selinuxvariant}
  install -p -m 644 SELinux/%{modulename}.pp.${selinuxvariant} \
    %{buildroot}%{_datadir}/selinux/${selinuxvariant}/%{modulename}.pp
done
/usr/sbin/hardlink -cv %{buildroot}%{_datadir}/selinux

%clean
rm -rf %{buildroot}

%preun
automation-controller-service stop

%pre server
getent group awx >/dev/null || groupadd -r awx
getent passwd awx >/dev/null || \
    useradd -r -g awx -d /var/lib/awx -s /bin/bash awx
# Remove old links
rm -f %{distro_python_sitelib}/sos/plugins/tower.py >/dev/null 2>&1 || :
rm -f %{distro_python_sitelib}/sos/report/plugins/tower.py >/dev/null 2>&1 || :

%post server
# Generate or reuse key file
if [ ! -f /etc/tower/SECRET_KEY ]; then
python3.8 -c 'import base64, os; f = open("/etc/tower/SECRET_KEY", "w"); f.write(base64.encodebytes(os.urandom(33)).decode().rstrip()); f.close()'
fi
chown root:awx /etc/tower/SECRET_KEY
chmod 640 /etc/tower/SECRET_KEY

# Create sosreport symlinks
ln -s /usr/share/sosreport/sos/plugins/controller.py  %{distro_python_sitelib}/sos/plugins/ || :
ln -s /usr/share/sosreport/sos/plugins/controller.py  %{distro_python_sitelib}/sos/report/plugins/ || :

# allow needed nginx access
for selinuxvariant in %{selinux_variants} ; do
    semodule -s ${selinuxvariant} -i %{_datadir}/selinux/${selinuxvariant}/%{modulename}.pp
done
/sbin/fixfiles -R %{modulename} restore || :
/sbin/restorecon -R /var/lib/awx/venv /var/lib/awx/job_status /var/run/tower
chown -R nginx:nginx /var/run/tower || :
if [ $1 -eq 1 ] ; then
        # Initial installation
        systemctl --no-reload preset automation-controller.service >/dev/null 2>&1 || :
fi

%preun server
if [ $1 -eq 0 ] ; then
        # Package removal, not upgrade
        systemctl --no-reload disable --now automation-controller.service >/dev/null 2>&1 || :
fi

%postun server
sed -i -e '/^# START AWX/,/^# END AWX/d' /etc/supervisord.conf
# Remove custom policy
if [ $1 -eq 0 ] ; then
    for selinuxvariant in %{selinux_variants} ; do
        /usr/sbin/semodule -s ${selinuxvariant} -r %{modulename} 2> /dev/null || :
    done
    # Remove sosreport symlinks
    rm -f %{distro_python_sitelib}/sos/plugins/controller.py || :
    rm -f %{distro_python_sitelib}/sos/report/plugins/controller.py || :
fi

%pre venv-tower
systemctl stop supervisord >/dev/null 2>&1 || :
getent group awx >/dev/null || groupadd -r awx
getent passwd awx >/dev/null || \
    useradd -r -g awx -d /var/lib/awx -s /bin/bash awx
python3.8 -m venv --clear /var/lib/awx/venv/awx

%files
%defattr(-,root,root)

%files server
%exclude /var/lib/awx/setup
%exclude /var/lib/awx/venv
%defattr(-,root,root)
%doc %{_datadir}/doc/ansible-tower
%{_bindir}/awx-manage
%{_bindir}/awx-python
%{_bindir}/tower-manage
%{_bindir}/tower-python
%{_bindir}/automation-controller-service
%{_bindir}/failure-event-handler
/lib/systemd/system/automation-controller.service
%attr(2775,nginx,nginx) /var/run/tower
%dir %attr(0755,awx,awx) /var/lib/awx
%dir %attr(0755,awx,awx) /var/lib/awx/rsyslog
%attr(0750,awx,awx) /var/log/tower
/var/lib/awx/[^j]*
%attr(0750,awx,awx) /var/lib/awx/rsyslog/rsyslog.conf
%attr(0750,awx,awx) /var/lib/awx/job_status
%{_datadir}/selinux/*/%{modulename}.pp
%{_datadir}/awx/
%{_datadir}/sosreport/sos/plugins/controller.py*
%config(noreplace) %attr(-,root,awx) %{_sysconfdir}/tower

%files ui
%defattr(-,root,root)
/var/lib/awx/venv/awx/lib/python3.8/site-packages/awx/static

%files setup
%defattr(-,root,root)
/var/lib/awx/setup
%{_bindir}/ansible-tower-setup

%files venv-tower
%exclude /var/lib/awx/venv/awx/lib/python3.8/site-packages/awx/static
%exclude /var/lib/awx/venv/awx/lib/python3.8/site-packages/awx/ui/static
%defattr(-,root,root)
/var/lib/awx/venv/awx

%files cli
%exclude /usr/bin/akit
/usr/bin/awx
%{_prefix}/lib/python3.8/site-packages/awxkit*

%changelog
* Wed Jul 07 2021 Satoe Imaishi <simaishi@redhat.com> - 4.0.0-1
- 4.0.0 release

* Mon May 18 2020 Shane McDonald <shanemcd@redhat.com>
- 3.7.0 release

* Wed Apr 22 2020 Shane McDonald <shanemcd@redhat.com>
- 3.6.4 release

* Wed Apr 22 2020 Shane McDonald <shanemcd@redhat.com>
- 3.5.6 release

* Thu Feb 20 2020 Yanis Guenane <yguenane@redhat.com>
- 3.6.3 release

* Thu Feb 20 2020 Yanis Guenane <yguenane@redhat.com>
- 3.5.5 release

* Mon Dec 16 2019 Shane McDonald <shanemcd@redhat.com>
- 3.6.2 release

* Mon Dec 16 2019 Shane McDonald <shanemcd@redhat.com>
- 3.5.4 release

* Mon Nov 25 2019 Shane McDonald <shanemcd@redhat.com>
- 3.6.1 release

* Wed Nov 13 2019 Shane McDonald <shanemcd@redhat.com>
- 3.6.0 release

* Thu Sep 19 2019 Shane McDonald <shanemcd@redhat.com>
- 3.5.3 release

* Wed Aug 14 2019 Shane McDonald <shanemcd@redhat.com>
- 3.5.2 release

* Wed Jul 10 2019 Shane McDonald <shanemcd@redhat.com>
- 3.5.1 release

* Wed May 29 2019 Shane McDonald <shanemcd@redhat.com>
- 3.5.0 release

* Tue Mar 26 2019 Shane McDonald <shanemcd@redhat.com>
- 3.4.3 release

* Wed Mar 6 2019 Shane McDonald <shanemcd@redhat.com>
- 3.4.2 release

* Thu Jan 24 2019 Shane McDonald <shanemcd@redhat.com>
- 3.4.1 release

* Thu Jan 24 2019 Shane McDonald <shanemcd@redhat.com>
- 3.3.4 release

* Wed Jan 9 2019 Shane McDonald <shanemcd@redhat.com>
- 3.4.0 release

* Tue Dec 18 2018 Shane McDonald <shanemcd@redhat.com>
- 3.3.3 release

* Thu Dec 6 2018 Shane McDonald <shanemcd@redhat.com>
- 3.3.2 release

* Thu Oct 25 2018 Shane McDonald <shanemcd@redhat.com>
- 3.3.1 release

* Wed Sep 12 2018 Shane McDonald <shanemcd@redhat.com>
- 3.3.0 release

* Fri Aug 10 2018 Shane McDonald <shanemcd@redhat.com>
- 3.2.6 release

* Fri May 11 2018 Shane McDonald <shanemcd@redhat.com>
- 3.2.5 release

* Thu Apr 5 2018 Shane McDonald <shanemcd@redhat.com>
- 3.2.4 release

* Tue Feb 20 2018 Shane McDonald <shanemcd@redhat.com>
- 3.2.3 release

* Tue Feb 20 2018 Shane McDonald <shanemcd@redhat.com>
- 3.2.2 release

* Thu Dec 7 2017 Shane McDonald <shanemcd@redhat.com>
- 3.2.2 release

* Wed Oct 11 2017 Shane McDonald <shanemcd@redhat.com>
- 3.2.1 release

* Sun Oct 1 2017 Shane McDonald <shanemcd@redhat.com>
- 3.2.0 release

* Sun Sep 17 2017 Shane McDonald <shanemcd@redhat.com>
- 3.1.5 release

* Mon Jul 10 2017 Shane McDonald <shanemcd@redhat.com>
- 3.1.4 release

* Mon May 1 2017 Matthew Jones <matburt@redhat.com>
- 3.1.3 release

* Fri Apr 7 2017 Matthew Jones <matburt@redhat.com>
- 3.1.2-2 release

* Fri Mar 31 2017 Matthew Jones <matburt@redhat.com>
- 3.1.2 release

* Tue Mar 28 2017 Matthew Jones <matburt@redhat.com>
- 3.1.1 release

* Mon Feb 27 2017 Matthew Jones <matburt@redhat.com>
- 3.1.0 release

* Fri Oct 28 2016 Matthew Jones <matburt@redhat.com>
- 3.0.3 release

* Wed Aug 31 2016 Matthew Jones <matburt@redhat.com>
- 3.0.2 release

* Fri Jul 29 2016 Matthew Jones <matburt@redhat.com>
- 3.0.1 release

* Tue Jul 19 2016 Matthew Jones <matburt@redhat.com>
- 3.0.0 release

* Wed Apr 20 2016 Matthew Jones <matburt@redhat.com>
- 2.4.5 release

* Fri Feb 19 2016 Matthew Jones <matburt@redhat.com>
- 2.4.4 release

* Wed Dec 16 2015 Matthew Jones <mjones@ansible.com>
- 2.4.3 release

* Wed Dec 9 2015 Matthew Jones <mjones@ansible.com>
- 2.4.2 release

* Thu Nov 19 2015 Matthew Jones <mjones@ansible.com>
- 2.4.1 release

* Wed Nov 18 2015 Matthew Jones <mjones@ansible.com>
- 2.4.0 release

* Fri Oct 2 2015 Matthew Jones <mjones@ansible.com>
- 2.3.1 release

* Tue Sep 22 2015 Matthew Jones <mjones@ansible.com>
- 2.3.0 release

* Wed Aug 19 2015 Matthew Jones <mjones@ansible.com>
- 2.2.2 release

* Wed Aug 12 2015 Matthew Jones <mjones@ansible.com>
- 2.2.1 release

* Tue Jul 14 2015 Matthew Jones <mjones@ansible.com>
- 2.2.0 release

* Tue Jun 23 2015 Chris Meyers <cmeyers@ansible.com>
- 2.1.6 release

* Fri Jun 12 2015 Chris Meyers <cmeyers@ansible.com>
- 2.1.5 release

* Fri May 1 2015 Matthew Jones <mjones@ansible.com>
- 2.1.4 release

* Tue Apr 14 2015 Matthew Jones <mjones@ansible.com>
- 2.1.3 release

* Wed Mar 25 2015 Matthew Jones <mjones@ansible.com>
- 2.1.2 release

* Wed Feb 04 2015 Matthew Jones <mjones@ansible.com>
- 2.1.1 release

* Thu Jan 08 2015 Matthew Jones <mjones@ansible.com>
- 2.1.0 release

* Wed Dec 10 2014 Matthew Jones <mjones@ansible.com>
- 2.0.5 release

* Fri Nov 21 2014 James Laska <jlaska@ansible.com>
- 2.0.4 release

* Fri Nov 14 2014 James Laska <jlaska@ansible.com>
- 2.0.3 release

* Mon Oct 6 2014 James Laska <jlaska@ansible.com>
- 2.0.2 release

* Thu Sep 4 2014 James Laska <jlaska@ansible.com>
- 2.0.1 release

* Tue Aug 19 2014 James Laska <jlaska@ansible.com>
- 2.0.0 release

* Mon Jul 28 2014 James Laska <jlaska@ansible.com>
- 1.4.12 release

* Sun May 11 2014 James Laska <jlaska@ansible.com>
- 1.4.11 release

* Wed Apr 23 2014 James Laska <jlaska@ansible.com>
- 1.4.10 release

* Thu Apr 17 2014 James Laska <jlaska@ansible.com>
- 1.4.9 release

* Tue Apr 8 2014 James Laska <jlaska@ansible.com>
- 1.4.8 release

* Wed Jan 29 2014 James Laska <jlaska@ansible.com>
- 1.4.5 release

* Mon Jun 10 2013 James Cammarata <jimi@sngx.net>
- Initial RPM packaging for awx
